/*

  This file is a part of GRALESHELL, a program to facilitate the simulation
  and inversion of gravitational lenses, based on the GRALE library.

  Copyright (C) 2008-2012 Jori Liesenborgs

  Contact: jori.liesenborgs@gmail.com
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  
*/

#include "cmdcosmology.h"
#include "graleshellinstance.h"
#include <shellp/shellcommand.h>
#include <shellp/shellcmdrealarg.h>
#include <shellp/shellcmdstringarg.h>
#include <shellp/iosystem.h>
#include <grale/cosmology.h>
#include <stdio.h>

#include "debugnew.h"

using namespace shellp;

CmdCosmologyShow::CmdCosmologyShow(const std::string &cmdName) : ShellCommand(cmdName)
{
	setDescription("Shows the current cosmological model parameters.");
}

CmdCosmologyShow::~CmdCosmologyShow()
{
}

bool CmdCosmologyShow::execute()
{
	GraleShellInstance *pGSInst = (GraleShellInstance *)ShellInstance::getInstance();
	IOSystem *pIOSys = pGSInst->getIOSystem();
	grale::Cosmology *pCosm = pGSInst->GetCosmology();
	char str[1024];

	pIOSys->writeOutputLine("Current cosmological parameters:");

	sprintf(str, "  H0      = %g * 100 km s^-1 Mpc^-1", pCosm->getH());
	pIOSys->writeOutputLine(str);
	sprintf(str, "  Omega_m = %g", pCosm->getMatter());
	pIOSys->writeOutputLine(str);
	sprintf(str, "  Omega_r = %g", pCosm->getRadiation());
	pIOSys->writeOutputLine(str);
	sprintf(str, "  Omega_v = %g", pCosm->getVacuum());
	pIOSys->writeOutputLine(str);
	sprintf(str, "  w       = %g", pCosm->getVacuumEOSParameter());
	pIOSys->writeOutputLine(str);

	return true;
}

CmdCosmologySet::CmdCosmologySet(const std::string &cmdName) : ShellCommand(cmdName)
{
	m_pHArg = new ShellCmdRealArg("h");
	m_pHArg->setDefault("0.71");
	m_pHArg->setDescription("Value of the Hubble constant H0, expressed in units of 100 km s^-1 Mpc^-1.");
	addArgument(m_pHArg);
	m_pWmArg = new ShellCmdRealArg("omega_m");
	m_pWmArg->setDefault("0.27");
	m_pWmArg->setDescription("Matter density parameter.");
	addArgument(m_pWmArg);
	m_pWrArg = new ShellCmdRealArg("omega_r");
	m_pWrArg->setDefault("0");
	m_pWrArg->setDescription("Radiation density parameter.");
	addArgument(m_pWrArg);
	m_pWvArg = new ShellCmdRealArg("omega_v");
	m_pWvArg->setDefault("0.73");
	m_pWvArg->setDescription("Vacuum density parameter.");
	addArgument(m_pWvArg);
	m_pWArg = new ShellCmdRealArg("w");
	m_pWArg->setDefault("-1");
	m_pWArg->setDescription("Parameter for the equation of state of the vacuum.");
	addArgument(m_pWArg);
	setDescription("Sets the current cosmological model parameters.");
}

CmdCosmologySet::~CmdCosmologySet()
{
	delete m_pHArg;
	delete m_pWmArg;
	delete m_pWrArg;
	delete m_pWvArg;
	delete m_pWArg;
}

bool CmdCosmologySet::execute()
{
	GraleShellInstance *pGSInst = (GraleShellInstance *)ShellInstance::getInstance();

	grale::Cosmology *pNewCosm = new grale::Cosmology(m_pHArg->getValue(), m_pWmArg->getValue(), m_pWrArg->getValue(), m_pWvArg->getValue(), m_pWArg->getValue());

	pGSInst->SetCosmology(pNewCosm);

	return true;
}


