/*

  This file is a part of GRALESHELL, a program to facilitate the simulation
  and inversion of gravitational lenses, based on the GRALE library.

  Copyright (C) 2008-2012 Jori Liesenborgs

  Contact: jori.liesenborgs@gmail.com
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  
*/

#include "cmdlensplane.h"
#include "graleshellinstance.h"
#include "graleshelllensplane.h"
#include <grale/real2dlensplanefunction.h>
#include <grale/real2dfunctiongnuplotplotter.h>
#include <grale/real2dfunctionfitsplotter.h>
#include <enut/ipv4address.h>

#include "debugnew.h"

using namespace shellp;

CmdLensPlane::CmdLensPlane(const std::string &cmdname) : ShellCommand(cmdname)
{
	x1arg = new ShellCmdRealArg("leftX");
	x1arg->setUnitType(UNIT_ANGULAR);
	x1arg->setDescription("X coordinate of the left side of the lens plane.");
	addArgument(x1arg);
	y1arg = new ShellCmdRealArg("bottomY");
	y1arg->setUnitType(UNIT_ANGULAR);
	y1arg->setDescription("Y coordinate of the bottom of the lens plane.");
	addArgument(y1arg);
	x2arg = new ShellCmdRealArg("rightX");
	x2arg->setUnitType(UNIT_ANGULAR);
	x2arg->setDescription("X coordinate of the right side of the lens plane.");
	addArgument(x2arg);
	y2arg = new ShellCmdRealArg("topY");
	y2arg->setUnitType(UNIT_ANGULAR);
	y2arg->setDescription("Y coordinate of the top of the lens plane.");
	addArgument(y2arg);
	numxarg = new ShellCmdIntArg("numX","512");
	numxarg->setMin(16);
	numxarg->setDescription("Number of points to sample the lens plane in the X direction.");
	addArgument(numxarg);
	numyarg = new ShellCmdIntArg("numY","512");
	numyarg->setMin(16);
	numyarg->setDescription("Number of points to sample the lens plane in the Y direction.");
	addArgument(numyarg);
}

CmdLensPlane::~CmdLensPlane()
{
	delete x1arg;
	delete x2arg;
	delete y1arg;
	delete y2arg;
	delete numxarg;
	delete numyarg;
}

CmdLensPlaneLocal::CmdLensPlaneLocal(const std::string &cmdname) : CmdLensPlane(cmdname)
{
	setDescription("Create lens plane based upon the current lens. Calculations are done locally.");
}

CmdLensPlaneLocal::~CmdLensPlaneLocal()
{
}

bool CmdLensPlaneLocal::execute()
{
	GraleShellInstance *gsinst = (GraleShellInstance *)ShellInstance::getInstance();
	grale::GravitationalLens *lens;
	
	if ((lens = gsinst->GetLens()) == 0)
	{
		setErrorString("A lens must have been created first");
		return false;
	}

	grale::Vector2D<double> bottomleft(GetX1(),GetY1());
	grale::Vector2D<double> topright(GetX2(),GetY2());
	
	GraleShellLensPlane *lensplane = new GraleShellLensPlane(lens);

	if (!lensplane->init(bottomleft,topright,GetNumX(),GetNumY(), gsinst->GetOpenCLKernel()))
	{
		setErrorString(std::string("Couldn't initialize the lens plane: ") + lensplane->getErrorString());
		delete lensplane;
		return false;
	}
	
	gsinst->SetLensPlane(lensplane);
	
	return true;
}

CmdLensPlaneRemote::CmdLensPlaneRemote(const std::string &cmdname) : CmdLensPlane(cmdname)
{
	ipstrarg = new ShellCmdStringArg("IPaddress");
	ipstrarg->setDescription("IP address of the rendering server.");
	addArgument(ipstrarg);
	portarg = new ShellCmdIntArg("port");
	portarg->setDescription("Port number of the rendering server.");
	portarg->setMin(1);
	portarg->setMax(65535);
	addArgument(portarg);
	setDescription("Create lens plane based upon the current lense. Calculations are done using a remote rendering server.");
}

CmdLensPlaneRemote::~CmdLensPlaneRemote()
{
	delete ipstrarg;
	delete portarg;
}

bool CmdLensPlaneRemote::execute()
{
	GraleShellInstance *gsinst = (GraleShellInstance *)ShellInstance::getInstance();
	grale::GravitationalLens *lens;
	
	if ((lens = gsinst->GetLens()) == 0)
	{
		setErrorString("A lens must have been created first");
		return false;
	}

	grale::Vector2D<double> bottomleft(GetX1(),GetY1());
	grale::Vector2D<double> topright(GetX2(),GetY2());
	
	nut::IPv4Address address;

	if (!address.setAddress(ipstrarg->getValue()))
	{
		setErrorString("An invalid IP address was entered");
		return false;
	}
	uint16_t port = (uint16_t)(portarg->getValue());
	
	GraleShellLensPlane *lensplane = new GraleShellLensPlane(lens);

	if (!lensplane->init(bottomleft, topright, GetNumX(), GetNumY(), address, port))
	{
		setErrorString(std::string("Couldn't initialize the image plane: ") + lensplane->getErrorString());
		delete lensplane;
		return false;
	}
	
	gsinst->SetLensPlane(lensplane);
	
	return true;
}

CmdLensPlaneSave::CmdLensPlaneSave(const std::string &cmdName) : ShellCommand(cmdName)
{
	m_pFileArg = new ShellCmdStringArg("filename");
	m_pFileArg->setDescription("Name of the file to which the lens plane should be saved.");
	addArgument(m_pFileArg);

	setDescription("Save the current lens plane to a file.");
}

CmdLensPlaneSave::~CmdLensPlaneSave()
{
	delete m_pFileArg;
}

bool CmdLensPlaneSave::execute()
{
	GraleShellInstance *pGSInst = (GraleShellInstance *)ShellInstance::getInstance();
	grale::LensPlane *pLensPlane = pGSInst->GetLensPlane();

	if (pLensPlane == 0)
	{
		setErrorString("No lens plane has been created yet");
		return false;
	}

	if (!pLensPlane->save(m_pFileArg->getValue()))
	{
		setErrorString(std::string("Couldn't save lens plane to the specified file: ") + pLensPlane->getErrorString());
		return false;
	}

	return true;
}

CmdLensPlaneLoad::CmdLensPlaneLoad(const std::string &cmdName) : ShellCommand(cmdName)
{
	m_pFileArg = new ShellCmdStringArg("filename");
	m_pFileArg->setDescription("Name of the file from which the lens plane should be loaded.");
	addArgument(m_pFileArg);

	setDescription("Load the lens plane from a file.");
}

CmdLensPlaneLoad::~CmdLensPlaneLoad()
{
	delete m_pFileArg;
}

bool CmdLensPlaneLoad::execute()
{
	GraleShellInstance *pGSInst = (GraleShellInstance *)ShellInstance::getInstance();
	grale::LensPlane *pLensPlane = 0;
	std::string errStr;

	if (!grale::LensPlane::load(m_pFileArg->getValue(), &pLensPlane, errStr))
	{
		setErrorString(std::string("Couldn't load lens plane from the specified file: ") + errStr);
		return false;
	}

	pGSInst->SetLensPlane(pLensPlane);

	return true;
}

CmdLensPlaneScale::CmdLensPlaneScale(const std::string &cmdName) : ShellCommand(cmdName)
{
	m_pScaleArg = new ShellCmdRealArg("scalefactor");
	m_pScaleArg->setDescription("Specifies the factor to scale the deflections with.");
	addArgument(m_pScaleArg);

	setDescription("Scales the deflections in the current lens plane.");
}

CmdLensPlaneScale::~CmdLensPlaneScale()
{
	delete m_pScaleArg;
}

bool CmdLensPlaneScale::execute()
{
	GraleShellInstance *pGSInst = (GraleShellInstance *)ShellInstance::getInstance();
	grale::LensPlane *pLensPlane = pGSInst->GetLensPlane();

	if (pLensPlane == 0)
	{
		setErrorString("No lens plane has been created yet");
		return false;
	}

	pLensPlane->scaleDeflections(m_pScaleArg->getValue());
	return true;
}

CmdLensPlanePlot::CmdLensPlanePlot(const std::string &cmdName) : ShellCommand(cmdName)
{
	m_pFileNameArg = new ShellCmdStringArg("filename");
	m_pFileNameArg->setDescription("Output file name.");
	addArgument(m_pFileNameArg);
	m_pX1Arg = new ShellCmdRealArg("leftX");
	m_pX1Arg->setUnitType(UNIT_ANGULAR);
	m_pX1Arg->setDescription("X coordinate of the left side of the plot area.");
	addArgument(m_pX1Arg);
	m_pY1Arg = new ShellCmdRealArg("bottomY");
	m_pY1Arg->setUnitType(UNIT_ANGULAR);
	m_pY1Arg->setDescription("Y coordinate of the bottom of the plot area.");
	addArgument(m_pY1Arg);
	m_pX2Arg = new ShellCmdRealArg("rightX");
	m_pX2Arg->setUnitType(UNIT_ANGULAR);
	m_pX2Arg->setDescription("X coordinate of the right side of the plot area.");
	addArgument(m_pX2Arg);
	m_pY2Arg = new ShellCmdRealArg("topY");
	m_pY2Arg->setUnitType(UNIT_ANGULAR);
	m_pY2Arg->setDescription("Y coordinate of the left side of the plot area.");
	addArgument(m_pY2Arg);
	m_pNumXArg = new ShellCmdIntArg("numX");
	m_pNumXArg->setMin(16);
	m_pNumXArg->setDescription("Number of points to sample the plot area in the X direction.");
	addArgument(m_pNumXArg);
	m_pNumYArg = new ShellCmdIntArg("numY");
	m_pNumYArg->setMin(16);
	m_pNumYArg->setDescription("Number of points to sample the plot area in the Y direction.");
	addArgument(m_pNumYArg);
}

CmdLensPlanePlot::~CmdLensPlanePlot()
{
	delete m_pFileNameArg;
	delete m_pX1Arg;
	delete m_pX2Arg;
	delete m_pY1Arg;
	delete m_pY2Arg;
	delete m_pNumXArg;
	delete m_pNumYArg;
}

CmdLensPlanePlotGnuplot::CmdLensPlanePlotGnuplot(const std::string &cmdName) : CmdLensPlanePlot(cmdName)
{
	m_pNumXArg->setDefault("75");
	m_pNumYArg->setDefault("75");
	
	m_pArg3D = new ShellCmdBoolArg("3d", "yes");
	m_pArg3D->setDescription("Flag indicating if a 3D plot should be generated.");
	addArgument(m_pArg3D);
	m_pColorArg = new ShellCmdBoolArg("color", "yes");
	m_pColorArg->setDescription("Flag indicating if color output should be generated.");
	addArgument(m_pColorArg);
	m_pXLabelArg = new ShellCmdStringArg("Xlabel", "X");
	m_pXLabelArg->setDescription("Label of the X axis.");
	addArgument(m_pXLabelArg);
	m_pYLabelArg = new ShellCmdStringArg("Ylabel", "Y");
	m_pYLabelArg->setDescription("Label of the Y axis.");
	addArgument(m_pYLabelArg);
	setDescription("Create a gnuplot file of the mass density based upon the information in the lensplane.");
}

CmdLensPlanePlotGnuplot::~CmdLensPlanePlotGnuplot()
{
	delete m_pArg3D;
	delete m_pColorArg;
	delete m_pXLabelArg;
	delete m_pYLabelArg;
}

bool CmdLensPlanePlotGnuplot::execute()
{
	GraleShellInstance *pGSInst = (GraleShellInstance *)ShellInstance::getInstance();
	grale::LensPlane *pLensPlane = pGSInst->GetLensPlane();
	
	if (pLensPlane == 0)
	{
		setErrorString("A lensplane must have been created first");
		return false;
	}

	grale::Real2DLensPlaneFunction f(*pLensPlane);
	grale::Real2DFunctionGnuPlotPlotter plotter(getFileName(), pGSInst->GetAngularUnit(), pGSInst->GetAngularUnit());
	
	plotter.setColor(m_pColorArg->getValue());
	plotter.setXLabel(m_pXLabelArg->getValue());
	plotter.setYLabel(m_pYLabelArg->getValue());
	plotter.set3D(m_pArg3D->getValue());
	
	grale::Vector2D<double> bottomLeft(getX1(), getY1());
	grale::Vector2D<double> topRight(getX2(), getY2());
	
	if (!plotter.plot(f, bottomLeft, topRight, getNumX(), getNumY()))
	{
		setErrorString(std::string("Error creating plot: ") + plotter.getErrorString());
		return false;
	}
	
	return true;
}

CmdLensPlanePlotFITS::CmdLensPlanePlotFITS(const std::string &cmdName) : CmdLensPlanePlot(cmdName)
{
	m_pNumXArg->setDefault("512");
	m_pNumYArg->setDefault("512");
	
	setDescription("Create a FITS plot of the mass density based upon the information in the lens plane.");
}

CmdLensPlanePlotFITS::~CmdLensPlanePlotFITS()
{
}

bool CmdLensPlanePlotFITS::execute()
{
#ifdef GRALECONFIG_SUPPORT_CFITSIO
	GraleShellInstance *pGSInst = (GraleShellInstance *)ShellInstance::getInstance();
	grale::LensPlane *pLensPlane = pGSInst->GetLensPlane();
	
	if (pLensPlane == 0)
	{
		setErrorString("A lensplane must have been created first");
		return false;
	}

	grale::Real2DLensPlaneFunction f(*pLensPlane);
	grale::Real2DFunctionFITSPlotter plotter(getFileName(), pGSInst->GetAngularUnit(), pGSInst->GetAngularUnit());
	
	grale::Vector2D<double> bottomLeft(getX1(), getY1());
	grale::Vector2D<double> topRight(getX2(), getY2());
	
	if (!plotter.plot(f, bottomLeft, topRight, getNumX(), getNumY()))
	{
		setErrorString(std::string("Error creating plot: ") + plotter.getErrorString());
		return false;
	}

	return true;
#else
	setErrorString("No cfitsio support was compiled in");
	return false;
#endif // GRALECONFIG_SUPPORT_CFITSIO
}

